
// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	///  This is a LinReg indicator plotting in three colors.  Basis is EMA_Colors indicator.
	///  v01 - 09/09/09 - zeller4 - Added slope and paint bars and selector from UI (User Interface).
    /// </summary>
	[Description("_Lin_Reg_Color_Paint_v01")]
	public class _Lin_Reg_Color_Paint_v01 : Indicator
	{
		#region Variables
		private int			period		= 10;
		private int			lrslope	= 0;
		private double		radToDegrees		= 180/Math.PI; // to convert Radians to Degrees for slope calc
		private int			angle1		= 25;
		private int			angle2		= 50;
		
		// Paint Bars
			private bool colorbars = false;
			private Color barColorUp = Color.Blue;
			private Color barColorDown = Color.Red;
			private Color barColorNeutral = Color.Yellow;
			private Color barColorOutline = Color.Black;
		
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(Color.Blue, "LinRegup"));
			Add(new Plot(Color.Red, "LinRegdown"));
			Add(new Plot(Color.Red, "LinRegslopedown"));
			Add(new Plot(Color.Blue, "LinRegslopeup"));
			Add(new Plot(Color.Gray, "LinRegflat"));
			Plots[0].Pen.Width = 3;
			Plots[1].Pen.Width = 3;
			Plots[2].Pen.Width = 3;
			Plots[3].Pen.Width = 3;
			Plots[4].Pen.Width = 3;
			Plots[0].Pen.DashStyle = DashStyle.Dash;
			Plots[1].Pen.DashStyle = DashStyle.Dash;
			Plots[2].Pen.DashStyle = DashStyle.Dash;
			Plots[3].Pen.DashStyle = DashStyle.Dash;
			Plots[4].Pen.DashStyle = DashStyle.Dash;
			Overlay				= true;
			PriceTypeSupported	= true;
			DisplayInDataBox	= false;
		}
		
		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			if(CurrentBar<Period) return;
				lrslope = (int)(radToDegrees*(Math.Atan((LinReg(Input,period)[0]-(LinReg(Input,period)[1]+LinReg(Input,period)[2])/2)/1.5/TickSize)));
			if(Rising(LinReg(Input,period)))
			{
				if(lrslope>=angle2)
				{
					
					
					LinRegslopeup.Set(1,LinReg(Input,period)[1]);
					LinRegslopeup.Set(LinReg(Input,period)[0]);
					if (ColorBars)
					{ 
						CandleOutlineColor = BarColorOutline; 
						BarColor = BarColorUp; 
					}
				}
				else
				if (lrslope>=angle1)
				{
					LinRegup.Set(1,LinReg(Input,period)[1]);
					LinRegup.Set(LinReg(Input,period)[0]);
					if (ColorBars)
					{ 
						CandleOutlineColor = BarColorOutline; 
						BarColor = BarColorUp; 
					}
				}
					else
					{	LinRegflat.Set(1,LinReg(Input,period)[1]);
						LinRegflat.Set(LinReg(Input,period)[0]);
						if (ColorBars)
						{ 
							CandleOutlineColor = BarColorOutline; 
							BarColor = BarColorNeutral; 
						}
					}
			}
			else
			{	if(lrslope<=-angle2)
				{
					LinRegslopedown.Set(1,LinReg(Input,period)[1]);
					LinRegslopedown.Set(LinReg(Input,period)[0]);
					if (ColorBars)
					{ 
						CandleOutlineColor = BarColorOutline; 
						BarColor = BarColorDown; 
					}
				}
				else
				if (lrslope<=-angle1)
				{
					LinRegdown.Set(1,LinReg(Input,period)[1]);
					LinRegdown.Set(LinReg(Input,period)[0]);
					if (ColorBars)
					{ 
						CandleOutlineColor = BarColorOutline; 
						BarColor = BarColorDown; 
					}
				}
					else
					{	LinRegflat.Set(1,LinReg(Input,period)[1]);
						LinRegflat.Set(LinReg(Input,period)[0]);
						if (ColorBars)
						{ 
							CandleOutlineColor = BarColorOutline; 
							BarColor = BarColorNeutral; 
						}
					}
			}
		}

		#region Properties
		/// <summary>
		/// </summary>
		
		[Description("Numbers of bars used for calculations")]
		[Category("Parameters")]
		public int Period
		{
			get { return period; }
			set { period = Math.Max(1, value); }
		}
		
		[Description("LinReg angle between Flat and Sloping zones.")]
		[Gui.Design.DisplayName("Angle 1")]
		[Category("Parameters")]
		public int Angle1
		{
			get { return angle1; }
			set { angle1 = Math.Max(1, value); }
		}

		[Description("LinReg angle between Sloping and Steep zones.")]
		[Gui.Design.DisplayName("Angle 2")]
		[Category("Parameters")]
		public int Angle2
		{
			get { return angle2; }
			set { angle2 = Math.Max(1, value); }
		}

		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LinRegup
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LinRegdown
        {
            get { return Values[1]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LinRegslopedown
        {
            get { return Values[2]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LinRegslopeup
        {
            get { return Values[3]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LinRegflat
        {
            get { return Values[4]; }
        }
/////////////////////////////////////////////////////////////////////////////////////		
		[Description("Color price bars?")]
        [Category("Visual")]
        [Gui.Design.DisplayName("01. Color Bars?")]
        public bool ColorBars
        {
            get { return colorbars; }
            set { colorbars = value; }
        }
		
        [XmlIgnore()]
        [Description("Color of up bars")]
        [Category("Visual")]
        [Gui.Design.DisplayNameAttribute("02. Up color")]
        public Color BarColorUp
        {
            get { return barColorUp; }
            set { barColorUp = value; }
        }

        [Browsable(false)]
        public string barColorUpSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(barColorUp); }
            set { barColorUp = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }

        [XmlIgnore()]
        [Description("Color of down bars")]
        [Category("Visual")]
        [Gui.Design.DisplayNameAttribute("03. Down color")]
        public Color BarColorDown
        {
            get { return barColorDown; }
            set { barColorDown = value; }
        }

        [Browsable(false)]
        public string barColorDownSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(barColorDown); }
            set { barColorDown = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        
        [XmlIgnore()]
        [Description("Color of neutral bars")]
        [Category("Visual")]
        [Gui.Design.DisplayNameAttribute("04. Neutral color")]
        public Color BarColorNeutral
        {
            get { return barColorNeutral; }
            set { barColorNeutral = value; }
        }

        [Browsable(false)]
        public string barColorNeutralSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(barColorNeutral); }
            set { barColorNeutral = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
		
        [XmlIgnore()]
        [Description("Color of bar outline")]
        [Category("Visual")]
        [Gui.Design.DisplayNameAttribute("05. Outline color")]
        public Color BarColorOutline
        {
            get { return barColorOutline; }
            set { barColorOutline = value; }
        }

        [Browsable(false)]
        public string barColorOutlineSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(barColorOutline); }
            set { barColorOutline = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
		
		
////////////////////////////////////////////////////////////////////////////////////////		

		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private _Lin_Reg_Color_Paint_v01[] cache_Lin_Reg_Color_Paint_v01 = null;

        private static _Lin_Reg_Color_Paint_v01 check_Lin_Reg_Color_Paint_v01 = new _Lin_Reg_Color_Paint_v01();

        /// <summary>
        /// _Lin_Reg_Color_Paint_v01
        /// </summary>
        /// <returns></returns>
        public _Lin_Reg_Color_Paint_v01 _Lin_Reg_Color_Paint_v01(int angle1, int angle2, int period)
        {
            return _Lin_Reg_Color_Paint_v01(Input, angle1, angle2, period);
        }

        /// <summary>
        /// _Lin_Reg_Color_Paint_v01
        /// </summary>
        /// <returns></returns>
        public _Lin_Reg_Color_Paint_v01 _Lin_Reg_Color_Paint_v01(Data.IDataSeries input, int angle1, int angle2, int period)
        {
            check_Lin_Reg_Color_Paint_v01.Angle1 = angle1;
            angle1 = check_Lin_Reg_Color_Paint_v01.Angle1;
            check_Lin_Reg_Color_Paint_v01.Angle2 = angle2;
            angle2 = check_Lin_Reg_Color_Paint_v01.Angle2;
            check_Lin_Reg_Color_Paint_v01.Period = period;
            period = check_Lin_Reg_Color_Paint_v01.Period;

            if (cache_Lin_Reg_Color_Paint_v01 != null)
                for (int idx = 0; idx < cache_Lin_Reg_Color_Paint_v01.Length; idx++)
                    if (cache_Lin_Reg_Color_Paint_v01[idx].Angle1 == angle1 && cache_Lin_Reg_Color_Paint_v01[idx].Angle2 == angle2 && cache_Lin_Reg_Color_Paint_v01[idx].Period == period && cache_Lin_Reg_Color_Paint_v01[idx].EqualsInput(input))
                        return cache_Lin_Reg_Color_Paint_v01[idx];

            _Lin_Reg_Color_Paint_v01 indicator = new _Lin_Reg_Color_Paint_v01();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Angle1 = angle1;
            indicator.Angle2 = angle2;
            indicator.Period = period;
            indicator.SetUp();

            _Lin_Reg_Color_Paint_v01[] tmp = new _Lin_Reg_Color_Paint_v01[cache_Lin_Reg_Color_Paint_v01 == null ? 1 : cache_Lin_Reg_Color_Paint_v01.Length + 1];
            if (cache_Lin_Reg_Color_Paint_v01 != null)
                cache_Lin_Reg_Color_Paint_v01.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cache_Lin_Reg_Color_Paint_v01 = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// _Lin_Reg_Color_Paint_v01
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator._Lin_Reg_Color_Paint_v01 _Lin_Reg_Color_Paint_v01(int angle1, int angle2, int period)
        {
            return _indicator._Lin_Reg_Color_Paint_v01(Input, angle1, angle2, period);
        }

        /// <summary>
        /// _Lin_Reg_Color_Paint_v01
        /// </summary>
        /// <returns></returns>
        public Indicator._Lin_Reg_Color_Paint_v01 _Lin_Reg_Color_Paint_v01(Data.IDataSeries input, int angle1, int angle2, int period)
        {
            return _indicator._Lin_Reg_Color_Paint_v01(input, angle1, angle2, period);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// _Lin_Reg_Color_Paint_v01
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator._Lin_Reg_Color_Paint_v01 _Lin_Reg_Color_Paint_v01(int angle1, int angle2, int period)
        {
            return _indicator._Lin_Reg_Color_Paint_v01(Input, angle1, angle2, period);
        }

        /// <summary>
        /// _Lin_Reg_Color_Paint_v01
        /// </summary>
        /// <returns></returns>
        public Indicator._Lin_Reg_Color_Paint_v01 _Lin_Reg_Color_Paint_v01(Data.IDataSeries input, int angle1, int angle2, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator._Lin_Reg_Color_Paint_v01(input, angle1, angle2, period);
        }

    }
}
#endregion
